@@ -80,7 +80,7 @@ DIAG_COSE_NON_OBJECT    =
 '无法 close 此类型的值。（除非给此类型设置 `__close` 元方法）'
 DIAG_COUNT_DOWN_LOOP    =
 '你的意思是 `{}` 吗？'
-DIAG_IMPLICIT_ANY       =
+DIAG_UNKNOWN            =
 '无法推测出类型。'
 DIAG_DEPRECATED         =
 '已废弃。'
@@ -94,7 +94,6 @@ DIAG_NOT_YIELDABLE      =
 '此函数的第 {} 个参数没有被标记为可让出，但是传入了异步函数。（使用 `---@param name async fun()` 来标记为可让出）'
 DIAG_DISCARD_RETURNS    =
 '不能丢弃此函数的返回值。'
-
 DIAG_CIRCLE_DOC_CLASS                 =
 '循环继承的类。'
 DIAG_DOC_FIELD_NO_CLASS               =
@@ -234,7 +233,6 @@ PARSER_INDEX_IN_FUNC_NAME =
 '命名函数的名称中不能使用 `[name]` 形式。'
 PARSER_UNKNOWN_ATTRIBUTE  =
 '局部变量属性应该是 `const` 或 `close`'
-
 PARSER_LUADOC_MISS_CLASS_NAME           =
 '缺少类名称。'
 PARSER_LUADOC_MISS_EXTENDS_SYMBOL       =
@@ -279,7 +277,6 @@ SYMBOL_ANONYMOUS        =
 
 HOVER_VIEW_DOCUMENTS    =
 '查看文档'
-
 HOVER_DOCUMENT_LUA51    =
 'http://www.lua.org/manual/5.1/manual.html#{}'
 HOVER_DOCUMENT_LUA52    =
@@ -290,7 +287,6 @@ HOVER_DOCUMENT_LUA54    =
 'http://www.lua.org/manual/5.4/manual.html#{}'
 HOVER_DOCUMENT_LUAJIT   =
 'http://www.lua.org/manual/5.1/manual.html#{}'
-
 HOVER_NATIVE_DOCUMENT_LUA51     =
 'command:extension.lua.doc?["en-us/51/manual.html/{}"]'
 HOVER_NATIVE_DOCUMENT_LUA52     =
@@ -301,7 +297,6 @@ HOVER_NATIVE_DOCUMENT_LUA54     =
 'command:extension.lua.doc?["en-us/54/manual.html/{}"]'
 HOVER_NATIVE_DOCUMENT_LUAJIT    =
 'command:extension.lua.doc?["en-us/51/manual.html/{}"]'
-
 HOVER_MULTI_PROTOTYPE      =
 '({} 个原型)'
 HOVER_STRING_BYTES         =
@@ -312,7 +307,6 @@ HOVER_MULTI_DEF_PROTO      =
 '({} 个定义，{} 个原型)'
 HOVER_MULTI_PROTO_NOT_FUNC =
 '({} 个非函数定义)'
-
 HOVER_USE_LUA_PATH      =
 '（搜索路径： `{}`）'
 HOVER_EXTENDS           =
@@ -321,6 +315,8 @@ HOVER_TABLE_TIME_UP     =
 '出于性能考虑，已禁用了部分类型推断。'
 HOVER_WS_LOADING        =
 '正在加载工作目录：{} / {}'
+HOVER_AWAIT_TOOLTIP     =
+'正在调用异步函数，可能会让出当前协程'
 
 ACTION_DISABLE_DIAG     =
 '在工作区禁用诊断 ({})。'
@@ -463,6 +459,10 @@ WINDOW_LUA_STATUS_TIP            =
 不是狗也不是狐狸！
              ↓↓↓
 ]]
+WINDOW_LUA_STATUS_DIAGNOSIS_TITLE=
+'进行工作区诊断'
+WINDOW_LUA_STATUS_DIAGNOSIS_MSG  =
+'是否进行工作区诊断？'
 WINDOW_APPLY_SETTING             =
 '应用设置'
 WINDOW_CHECK_SEMANTIC            =
@@ -489,6 +489,8 @@ WINDOW_APPLY_WHITOUT_SETTING     =
 '应用但不修改设置'
 WINDOW_ASK_APPLY_LIBRARY         =
 '是否需要将你的工作环境配置为 `{}` ？'
+WINDOW_SEARCHING_IN_FILES        =
+'正在文件中搜索...'
 
 CONFIG_LOAD_FAILED               =
 '无法读取设置文件：{}'
