@@ -19,7 +19,7 @@ arg = {}
 ---@return T
 function assert(v, message) end
 
----@alias cgopt
+---@alias gcoptions
 ---|>'"collect"'      # 做一次完整的垃圾收集循环。
 ---| '"stop"'         # 停止垃圾收集器的运行。
 ---| '"restart"'      # 重启垃圾收集器的自动运行。
@@ -34,7 +34,7 @@ function assert(v, message) end
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-collectgarbage"])
 ---
----@param opt? cgopt
+---@param opt? gcoptions
 ---@return any
 function collectgarbage(opt, ...) end
 
@@ -153,6 +153,12 @@ function loadfile(filename, mode, env) end
 ---@nodiscard
 function loadstring(text, chunkname) end
 
+---@version 5.1
+---@param proxy boolean|table
+---@return userdata
+---@nodiscard
+function newproxy(proxy) end
+
 ---@version 5.1
 ---
 ---创建一个模块。
@@ -382,8 +388,10 @@ function xpcall(f, msgh, arg1, ...) end
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-unpack"])
 ---
----@param list table
+---@generic T
+---@param list T[]
 ---@param i?   integer
 ---@param j?   integer
+---@return T
 ---@nodiscard
 function unpack(list, i, j) end
