@@ -9,7 +9,7 @@
 table = {}
 
 ---
----提供一个列表，其所有元素都是字符串或数字，返回字符串 `list[i]..sep..list[i+1] ··· sep..list[j]`。
+---Given a list where all elements are strings or numbers, returns the string `list[i]..sep..list[i+1] ··· sep..list[j]`.
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.concat"])
 ---
@@ -22,7 +22,7 @@ table = {}
 function table.concat(list, sep, i, j) end
 
 ---
----在 `list` 的位置 `pos` 处插入元素 `value`。
+---Inserts element `value` at position `pos` in `list`.
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.insert"])
 ---
@@ -34,7 +34,7 @@ function table.insert(list, pos, value) end
 
 ---@version <5.1
 ---
----返回给定表的最大正数索引，如果表没有正数索引，则返回零。
+---Returns the largest positive numerical index of the given table, or zero if the table has no positive numerical indices.
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.maxn"])
 ---
@@ -45,7 +45,7 @@ function table.maxn(table) end
 
 ---@version >5.3
 ---
----将元素从表 `a1` 移到表 `a2`。
+---Moves elements from table `a1` to table `a2`.
 ---```lua
 ---a2[t],··· =
 ---a1[f],···,a1[e]
@@ -103,10 +103,51 @@ function table.sort(list, comp) end
 ---
 ---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.unpack"])
 ---
----@param list table
+---@generic T
+---@param list T[]
 ---@param i?   integer
 ---@param j?   integer
+---@return T
 ---@nodiscard
 function table.unpack(list, i, j) end
 
+---@version <5.1, JIT
+---
+---遍历表中的每一个元素，并以key和value执行回调函数。如果回调函数返回一个非nil值则循环终止,并且返回这个值。该函数等同pair(list),比pair(list)更慢。不推荐使用
+---
+---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.foreach"])
+---
+---@generic T
+---@param list any
+---@param callback fun(key: string, value: any):T|nil
+---@return T|nil
+---@deprecated
+function table.foreach(list, callback) end
+
+---@version <5.1, JIT
+---
+---遍历数组中的每一个元素，并以索引号index和value执行回调函数。如果回调函数返回一个非nil值则循环终止,并且返回这个值。该函数等同ipair(list),比ipair(list)更慢。不推荐使用
+---
+---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.foreachi"])
+---
+---@generic T
+---@param list any
+---@param callback fun(key: string, value: any):T|nil
+---@return T|nil
+---@deprecated
+function table.foreachi(list, callback) end
+
+---@version <5.1, JIT
+---
+---返回表的长度。该函数等价于#list。
+---
+---[查看文档](command:extension.lua.doc?["en-us/54/manual.html/pdf-table.getn"])
+---
+---@generic T
+---@param list T[]
+---@return integer
+---@nodiscard
+---@deprecated
+function table.getn(list) end
+
 return table
