@@ -1,8 +1,8 @@
 local buildName   = require 'core.hover.name'
-local buildArg    = require 'core.hover.arg'
+local buildArgs   = require 'core.hover.args'
 local buildReturn = require 'core.hover.return'
 local buildTable  = require 'core.hover.table'
-local infer       = require 'core.infer'
+local infer       = require 'vm.infer'
 local vm          = require 'vm'
 local util        = require 'utility'
 local lang        = require 'language'
@@ -12,15 +12,15 @@ local guide       = require 'parser.guide'
 
 local function asFunction(source, oop)
     local name  = buildName(source, oop)
-    local arg   = buildArg(source, oop)
+    local args  = buildArgs(source)
     local rtn   = buildReturn(source)
     local lines = {}
 
     lines[1] = string.format('%s%s %s(%s)'
-        , vm.isAsync(source) and 'async ' or ''
-        , oop and 'method' or 'function'
+        , vm.isAsync(source) and '(async) ' or ''
+        , oop and '(method)' or 'function'
         , name or ''
-        , arg
+        , oop and table.concat(args, ', ', 2) or table.concat(args, ', ')
     )
     lines[2] = rtn
 
@@ -30,12 +30,11 @@ end
 local function asDocTypeName(source)
     local defs = vm.getDefs(source)
     for _, doc in ipairs(defs) do
-        if doc.type == 'doc.class.name' then
-            return 'class ' .. doc[1]
+        if doc.type == 'doc.class' then
+            return '(class) ' .. doc.class[1]
         end
-        if doc.type == 'doc.alias.name' then
-            local extends = doc.parent.extends
-            return lang.script('HOVER_EXTENDS', infer.searchAndViewInfers(extends))
+        if doc.type == 'doc.alias' then
+            return '(alias) ' .. doc.alias[1] .. ' ' .. lang.script('HOVER_EXTENDS', infer.getInfer(doc.extends):view())
         end
     end
 end
@@ -43,16 +42,10 @@ end
 ---@async
 local function asValue(source, title)
     local name    = buildName(source, false) or ''
-    local type    = infer.searchAndViewInfers(source)
-    local literal = infer.searchAndViewLiterals(source)
-    local cont
-    if  not infer.hasType(source, 'string')
-    and not type:find('%[%]$') then
-        if #vm.getRefs(source, '*') > 0
-        or infer.hasType(source, 'table') then
-            cont = buildTable(source)
-        end
-    end
+    local ifr     = infer.getInfer(source)
+    local type    = ifr:view()
+    local literal = ifr:viewLiterals()
+    local cont    = buildTable(source)
     local pack = {}
     pack[#pack+1] = title
     pack[#pack+1] = name .. ':'
@@ -62,6 +55,7 @@ local function asValue(source, title)
     if  cont
     and (  type == 'table'
         or type == 'any'
+        or type == 'unknown'
         or type == 'nil') then
         type = nil
     end
@@ -78,12 +72,28 @@ end
 
 ---@async
 local function asLocal(source)
-    return asValue(source, 'local')
+    local node
+    if source.type == 'local'
+    or source.type == 'self' then
+        node = source
+    else
+        node = source.node
+    end
+    if node.type == 'self' then
+        return asValue(source, '(self)')
+    end
+    if node.parent.type == 'funcargs' then
+        return asValue(source, '(parameter)')
+    elseif guide.getParentFunction(source) ~= guide.getParentFunction(node) then
+        return asValue(source, '(upvalue)')
+    else
+        return asValue(source, 'local')
+    end
 end
 
 ---@async
 local function asGlobal(source)
-    return asValue(source, 'global')
+    return asValue(source, '(global)')
 end
 
 local function isGlobalField(source)
@@ -118,24 +128,23 @@ local function asField(source)
     if isGlobalField(source) then
         return asGlobal(source)
     end
-    return asValue(source, 'field')
+    return asValue(source, '(field)')
 end
 
 local function asDocFieldName(source)
-    local name     = source[1]
-    local docField = source.parent
+    local name     = source.field[1]
     local class
-    for _, doc in ipairs(docField.bindGroup) do
+    for _, doc in ipairs(source.bindGroup) do
         if doc.type == 'doc.class' then
             class = doc
             break
         end
     end
-    local view = infer.searchAndViewInfers(docField.extends)
+    local view = infer.getInfer(source.extends):view()
     if not class then
-        return ('field ?.%s: %s'):format(name, view)
+        return ('(field) ?.%s: %s'):format(name, view)
     end
-    return ('field %s.%s: %s'):format(class.class[1], name, view)
+    return ('(field) %s.%s: %s'):format(class.class[1], name, view)
 end
 
 local function asString(source)
@@ -184,6 +193,7 @@ return function (source, oop)
     or     source.type == 'doc.type.function' then
         return asFunction(source, oop)
     elseif source.type == 'local'
+    or     source.type == 'self'
     or     source.type == 'getlocal'
     or     source.type == 'setlocal' then
         return asLocal(source)
@@ -205,7 +215,7 @@ return function (source, oop)
         return asNumber(source)
     elseif source.type == 'doc.type.name' then
         return asDocTypeName(source)
-    elseif source.type == 'doc.field.name' then
+    elseif source.type == 'doc.field' then
         return asDocFieldName(source)
     end
 end
